;Hbeam_pickup.asm
;-----------------------
;Standalone pickup for Hyper beam. Includes changes to commit the hyper beam flag to SRAM.
;Does not count toward item % with the vanilla counting routine
;Breaks the vanilla method of getting hyper (because the hyper beam flag has not been repointed for those cutscenes)
lorom
;Vanilla Instruction Defines
{
!Sleep = $86B4		;Do nothing until woken up by setting its instruction timer <> 0
!Delete = $86BC	    ;Delete PLM object
!SetPre = $86C1	    ;Preinst Pointer = [Y]
!ClrPre = $86CA	    ;Clear Pre-Instruction
!LnkIns = $8A24	    ;Set link instruction = [Y]
!SubInst = $8A3E	;Goto [Y], but store current [Y] as the link instruction
!RTSlist = $8A3A	;Go to link instruction
!CalFcn1 = $870B	;Call external function at [Y]
!CalFcn2 = $86EB	;Call external function at [Y] with [A] = [Y]+3
!SetBTS = $8AF1	    ;Set BTS at PLM coordinate to [Y], 8-bit operand.
!Goto = $8724		;Go to [Y]
!Step = $8729		;Adds value at [Y] onto [Y]. One byte operand. Seems hard to use.
!DecTime1 = $873F	;Decrement timer and goto [Y] if nonzero
!DecTime2 = $8747	;Decrement timer and step [Y] if nonzero
!SetTime1 = $874E	;Set timer = [Y], 8-bit operand
!SetTime2 = $875A	;Set timer = [Y], 16-bit operand
!DMAxfer = $87E5	;Transfer [[Y]] bytes from [[Y] + 2] to VRAM [[Y] + 5] using $D0 Table.
!BossChek = $880E	;Goto [[Y] + 1] if any of the boss bits [[Y]] are set
!SetBossB = $8821	;Set boss bits at [Y], 8-bit operand
!EventChek = $882D	;Goto [[Y] + 2] if the event [[Y]] is set
!EventSet = $883E	;Set event [Y]
!MusicCHG = $8BDD	;clear music queue and queue music track [[Y]]
!Lib1snd = $8C07	;queue sound [[Y]], sound library 1, max queued sounds allowed = 6
!Lib2snd = $8C10	;queue sound [[Y]], sound library 2, max queued sounds allowed = 6
!Lib3snd = $8C19	;queue sound [[Y]], sound library 3, max queued sounds allowed = 6
!GotoProx = $8D41	;go to [[Y] + 2] if Samus is within [[Y]] columns and [[Y] + 1] rows of PLM
!MovDown = $8D89	;Move PLM down one block
}


!HeaderFree = $84ABE3	;Replace some MB-related PLMs. Free space until $AC89
!CodeFree = $84EFD3		;Free space at end of bank. Will probably need to move this for compatibility with Explodoors...
!89Free = $89AEFD		;Space for plm graphics

!HbeamFlag = $09EC		;SRAM to store the new flag in. This one is at the end of the vanilla Samus properties like missile count etc.

org !HeaderFree   ;freespace in $84
HEADERS:
print pc, " - Hbeam"
dw $EE64, Hbeam_inst_initial

org !CodeFree	;freespace in $84
Hbeam:
{
.inst
..initial
dw $8764,ITEMGFX_Hbeam : db $00,$00,$00,$00,$00,$01,$00,$00	;plasma beam gfx for now
dw $887C, ..delete
dw !LnkIns, ..collected
dw !SetPre, $DF89
...loop1
dw $E04F	;item frame 1
dw $E067	;item frame 2
dw !Goto, ...loop1
..collected
dw $8899
dw $8BDD : db $02	;do fanfare
dw $88F3, $0000 : db $01 ;pick up equipment #$0000 [nothing] and show message box 1
dw .GetH
..delete
dw !Goto, $DFA9	;Draw air vanilla item instruction

.GetH
PHY : PHX
LDA #$1009 : STA $09A6	;Set beams to Charge, Wave, Plasma
LDA #$0001 : STA !HbeamFlag	;Activate hyper beam flag
LDY #$E1F0 : JSL $8DC4E9 ;Spawn hyper beam palette object
JSL $90AC8D	;Update beam tiles and palette
PLX : PLY
RTS

.SpawnPal
;Spawn the hyper beam palette object when loading in.
PHX : PHY
LDA !HbeamFlag : BEQ +
LDY #$E1F0 : JSL $8DC4E9
+
PLY : PLX
RTL
;Called from $828067
}


;Misc outside $84
org $90ADA9	;this one happens during door transitions
LDA !HbeamFlag

org $90BAFF      ;|
LDA !HbeamFlag   ;|;Relocate hyper beam flag for firing routine. If vanilla cutscenes were still used, they would need to also have this value changed from $0A76
org $90B816      ;|
LDA !HbeamFlag   ;|
org $90B887      ;|
LDA !HbeamFlag   ;|

org $91D794      ;also for firing
LDA !HbeamFlag

org $82A17B      ;|
LDA !HbeamFlag   ;|;For loading menus
org $82ABBA      ;|
LDA !HbeamFlag   ;|

org $82B48C      ;|;-controls letting you move to beams or not on the menu
LDA !HbeamFlag   ;|;Selecting beams with hyper turned on lets you equip/unequip the beams as normal
org $82B459      ;|;But the only net result is changing what beam tiles Hyper will use.
LDA !HbeamFlag   ;|;Wave/Plasma is the vanilla combo
org $82B046      ;|
LDA !HbeamFlag   ;|


org $828067	;overwrite a dead branch in the loading routine
JSL Hbeam_SpawnPal

;--GFX--------
org !89Free
;Custom PLM graphics
ITEMGFX:
.Hbeam
incbin hyperbeamPLM.gfx